@@ -78,36 +78,80 @@ class EC2VM(VMResource):
 
     def create(self, wait=True):
         self.is_created = False
+        # start with efa enabled if it is supported
+        self.efa_support = False
+        self.hibernation_support = False
+        try:
+            self.efa_support = self.client.describe_instance_types(
+                InstanceTypes=[self.instance_type],
+            )['InstanceTypes'][0]['NetworkInfo']['EfaSupported']
+            LOG.info('efa supported status: {}'.format(self.efa_support))
+        except Exception as error:
+            LOG.info('Cannot determin efa status, disable in launch')
+        try:
+            self.hibernation_support = self.client.describe_instance_types(
+                InstanceTypes=[self.instance_type],
+            )['InstanceTypes'][0]['HibernationSupported']
+            LOG.info('Hibernation supported status: {}'.format(self.hibernation_support))
+        except Exception as error:
+            LOG.info('Cannot determin Hibernation status, disable in launch')
+        self.root_device_name = '/dev/sda1'
+        try:
+            self.root_device_name = self.client.describe_images(ImageIds=[self.ami_id])['Images'][0]['RootDeviceName']
+            LOG.info('root device name: {}'.format(self.root_device_name))
+        except Exception as error:
+            LOG.info('Cannot determin root device name, use default {}'.format(self.root_device_name))
+
+        vm_kwargs = {
+            'BlockDeviceMappings':[
+                {
+                    'DeviceName': self.root_device_name,
+                    'Ebs': {
+                        'DeleteOnTermination': True,
+                        'VolumeSize': 10,
+                        # root disk must be encrypted when hibernation enabled
+                        'Encrypted': self.hibernation_support
+                    },
+                },
+            ],
+            "ImageId":self.ami_id,
+            "InstanceType":self.instance_type,
+            "KeyName":self.ssh_key_name,
+            "MaxCount":1,
+            "MinCount":1,
+            "NetworkInterfaces":[
+                {
+                    'AssociatePublicIpAddress': True,
+                    'DeleteOnTermination': True,
+                    'DeviceIndex': 0,
+                    'SubnetId': self.subnet_id,
+                    'Groups': [
+                         self.security_group_ids,
+                     ],
+                },
+            ],
+            "TagSpecifications":[
+                {
+                    'ResourceType': 'instance',
+                    'Tags': [
+                        {
+                            'Key': 'Name',
+                            'Value': self.tag
+                        },
+                    ]
+                },
+            ],
+            'HibernationOptions':{
+                'Configured': self.hibernation_support
+            },
+            "UserData":'#!/bin/bash\nmkdir /tmp/userdata_{}'.format(self.run_uuid)
+        }
+        if self.efa_support:
+            vm_kwargs["NetworkInterfaces"][0]["InterfaceType"] = 'efa'
+        #vm_kwargs["EnclaveOptions"]["Enabled"] = True
         if self.additionalinfo == None or self.additionalinfo == '':
             try:
-                self.ec2_instance = self.resource.create_instances(
-                    ImageId=self.ami_id,
-                    InstanceType=self.instance_type,
-                    KeyName=self.ssh_key_name,
-                    MaxCount=1,
-                    MinCount=1,
-                    NetworkInterfaces=[
-                        {
-                            'AssociatePublicIpAddress': True,
-                            'DeviceIndex': 0,
-                            'SubnetId': self.subnet_id,
-                            'Groups': [
-                                 self.security_group_ids,
-                             ],
-                        },
-                    ],
-                    TagSpecifications=[
-                        {
-                            'ResourceType': 'instance',
-                            'Tags': [
-                                {
-                                    'Key': 'Name',
-                                    'Value': self.tag
-                                },
-                            ]
-                        },
-                    ],
-                    UserData='#!/bin/bash\nmkdir /tmp/userdata_{}'.format(self.run_uuid))[0]
+                self.ec2_instance = self.resource.create_instances(**vm_kwargs)[0]
                 self.is_created = True
             except ClientError as err:
                 LOG.error("Failed to create instance!")
@@ -118,42 +162,12 @@ class EC2VM(VMResource):
         if self.additionalinfo != None and self.additionalinfo != '':
             for additionalinfo in self.additionalinfo.split(';'):
                 try:
-                    LOG.error("Create instance using addtionalinfo: {}".format(additionalinfo))
-                    self.ec2_instance = self.resource.create_instances(
-                        ImageId=self.ami_id,
-                        InstanceType=self.instance_type,
-                        KeyName=self.ssh_key_name,
-                        MaxCount=1,
-                        MinCount=1,
-                        AdditionalInfo=additionalinfo,
-                        NetworkInterfaces=[
-                            {
-                                'AssociatePublicIpAddress': True,
-                                'DeviceIndex': 0,
-                                'SubnetId': self.subnet_id,
-                                'Groups': [
-                                     self.security_group_ids,
-                                 ],
-                            },
-                        ],
-                        TagSpecifications=[
-                            {
-                                'ResourceType': 'instance',
-                                'Tags': [
-                                    {
-                                        'Key': 'Name',
-                                        'Value': self.tag
-                                    },
-                                ]
-                            },
-                        ],
-                        UserData='#!/bin/bash\nmkdir /home/%s/instance_create_%s' %
-                        (self.ssh_user, self.instance_type))[0]
+                    LOG.error("Create instance with AdditionalInfo: {}".format(additionalinfo))
+                    vm_kwargs['AdditionalInfo'] = additionalinfo
+                    self.ec2_instance = self.resource.create_instances(**vm_kwargs)[0]
                     self.is_created = True
-                except ClientError as err:
-                    LOG.error("Failed to create instance, try another addtionalinfo {}".format(err))
                 except Exception as err:
-                    LOG.error("Failed to create instance, try another addtionalinfo {}".format(err))
+                    LOG.error("Failed to create instance, try another AdditionalInfo {}".format(err))
                 if self.is_created:
                     break
             if not self.is_created:
@@ -166,6 +180,7 @@ class EC2VM(VMResource):
                 LOG.error("Failed to wait instance running! %s" % err)
 
         self.id = self.ec2_instance.id
+        self.show()
         # self.ipv4 = self.ec2_instance.public_ip_address
         self.floating_ip
         #self.boot_volume_id
@@ -203,12 +218,12 @@ class EC2VM(VMResource):
         if self.additionalinfo != None and self.additionalinfo != '':
             for additionalinfo in self.additionalinfo.split(';'):
                 try:
-                    LOG.info("Start instance using addtionalinfo: {}".format(additionalinfo))
+                    LOG.info("Start instance using AdditionalInfo: {}".format(additionalinfo))
                     self.ec2_instance.start(AdditionalInfo=additionalinfo)
                     start_ok = True
                     break
                 except Exception as err:
-                    LOG.error("Failed to start instance, try another addtionalinfo {}".format(err))
+                    LOG.error("Failed to start instance, try another AdditionalInfo: {}".format(err))
         if not start_ok:
             try:
                 self.ec2_instance.start()
@@ -229,10 +244,10 @@ class EC2VM(VMResource):
             self.floating_ip
         return True
 
-    def stop(self, wait=True, loops=4):
+    def stop(self, wait=True, loops=4, hibernate=False):
         try:
-            LOG.info("Stopping instance %s " % self.id)
-            self.ec2_instance.stop()
+            LOG.info("Stopping instance {} with hibernation {}".format(self.id, hibernate))
+            self.ec2_instance.stop(Hibernate=hibernate)
         except Exception as err:
             LOG.error("%s" % err)
             return False
@@ -301,8 +316,10 @@ class EC2VM(VMResource):
             return False
 
     def send_hibernation(self):
-        LOG.info("Wow, will add support later!")
-        return False
+        if not self.hibernation_support:
+            LOG.info("this instance not support hibernation")
+            return False
+        return self.stop(hibernate=True)
 
     def get_console_log(self, silient=False):
         ret = None
@@ -324,7 +341,7 @@ class EC2VM(VMResource):
 
     def get_state(self):
         try:
-            state = 'unknow'
+            state = 'unknown'
             self.ec2_instance.reload()
             state = self.ec2_instance.state['Name']
             LOG.info("instance is in {}".format(state))
@@ -381,13 +398,11 @@ class EC2VM(VMResource):
             return False
         return True
 
-    def attach_nic(self):
-        LOG.info("Wow, will add support later!")
-        return False
+    def attach_nic(self,nic,device_index=1, wait=True):
+        return nic.attach_to_instance(self.id, device_index, wait=wait)
 
-    def detach_nic(self):
-        LOG.info("Wow, will add support later!")
-        return False
+    def detach_nic(self,nic,force=False):
+        return nic.detach_from_instance(self.id, wait=True, force=force)
 
 class EC2Volume(StorageResource):
     '''
@@ -447,76 +462,29 @@ class EC2Volume(StorageResource):
             if self.type == 'sc1' and self.size < 500:
                 self.size = 500
                 LOG.info("{} minimal size is 500G, create 500G disk instead.".format(self.type))
-            if self.outpostarn is None:
-                if self.type == 'io1':
-                    self.volume = self.snapshot = self.resource.create_volume(
-                        AvailabilityZone=self.zone,
-                        Size=self.size,
-                        VolumeType=self.type,
-                        Iops=self.iops,
-                        TagSpecifications=[
+            disk_kwargs = {
+                "AvailabilityZone":self.zone,
+                "Size":self.size,
+                "VolumeType":self.type,
+                "Iops":self.iops,
+                "TagSpecifications":[
+                    {
+                        'ResourceType': 'volume',
+                        'Tags': [
                             {
-                                'ResourceType': 'volume',
-                                'Tags': [
-                                    {
-                                        'Key': 'Name',
-                                        'Value': self.tag
-                                    },
-                                ]
+                                'Key': 'Name',
+                                'Value': self.tag
                             },
-                        ])
-                else:
-                    self.volume = self.snapshot = self.resource.create_volume(
-                        AvailabilityZone=self.zone,
-                        Size=self.size,
-                        VolumeType=self.type,
-                        TagSpecifications=[
-                            {
-                                'ResourceType': 'volume',
-                                'Tags': [
-                                    {
-                                        'Key': 'Name',
-                                        'Value': self.tag
-                                    },
-                                ]
-                            },
-                        ])
-            else:
-                if self.disktype == 'io1':
-                    self.volume = self.snapshot = self.resource.create_volume(
-                        AvailabilityZone=self.zone,
-                        Size=self.size,
-                        VolumeType=self.type,
-                        Iops=self.iops,
-                        OutpostArn=self.outpostarn,
-                        TagSpecifications=[
-                            {
-                                'ResourceType': 'volume',
-                                'Tags': [
-                                    {
-                                        'Key': 'Name',
-                                        'Value': self.tag
-                                    },
-                                ]
-                            },
-                        ])
-                else:
-                    self.volume = self.snapshot = self.resource.create_volume(
-                        AvailabilityZone=self.zone,
-                        Size=self.size,
-                        VolumeType=self.type,
-                        OutpostArn=self.outpostarn,
-                        TagSpecifications=[
-                            {
-                                'ResourceType': 'volume',
-                                'Tags': [
-                                    {
-                                        'Key': 'Name',
-                                        'Value': self.tag
-                                    },
-                                ]
-                            },
-                        ])
+                        ]
+                    },
+                ]
+            }
+            if self.type != 'io1':
+                disk_kwargs.pop('Iops')
+            if self.outpostarn:
+                disk_kwargs['OutpostArn'] = self.outpostarn
+            LOG.info(disk_kwargs)
+            self.volume = self.snapshot = self.resource.create_volume(**disk_kwargs)
             self.volume.reload()
             self.id = self.volume.id
             timeout = 120
@@ -562,7 +530,7 @@ class EC2Volume(StorageResource):
 
     def get_state(self):
         try:
-            state = 'unknow'
+            state = 'unknown'
             self.volume.reload()
             state = self.volume.state
             LOG.info("disk state: {}".format(state))
@@ -592,3 +560,229 @@ class EC2Volume(StorageResource):
         except Exception as err:
             LOG.info("Volume does not exists %s" % self.id)
             return False
+
+class EC2NIC(NetworkResource):
+    '''
+    AWS Network class
+    '''
+    __network_interface = None
+
+    def __init__(self, params):
+        super(EC2NIC, self).__init__(params)
+        self.profile_name = params.get('profile_name')
+        if self.profile_name is None:
+            self.profile_name = 'default'
+        self.session = boto3.session.Session(profile_name=self.profile_name, region_name=params.get('region'))
+        self.__ec2 = self.session.resource('ec2', region_name=params.get('region'))
+        self._resource = self.session.resource('ec2', region_name=params.get('region'))
+        if params.get('ipv6'):
+            self.subnet_id = params.get('subnet_id_ipv6')
+            LOG.info('Instance support ipv6, use subnet %s', self.subnet_id)
+        else:
+            self.subnet_id = params.get('subnet_id_ipv4')
+            LOG.info('Instance only support ipv4, use subnet %s',
+                     self.subnet_id)
+        self.subnet = self.__ec2.Subnet(self.subnet_id)
+
+        self.zone = self.subnet.availability_zone
+        LOG.info("Get zone from current instance's subnet {}".format(self.zone))
+        if params.get('tagname'):
+            self.tag = params.get('tagname')
+        else:
+            self.tag = 'os_tests_network_ec2'
+        self.outpostarn = params.get('outpostarn')
+        self.id = None
+        self.security_group_ids = params.get('security_group_ids')
+
+    def show(self):
+        """
+        Show instance properties
+        Must show after VM properties are changed
+        """
+        if self.is_exist():
+            LOG.info("NIC ID: {}".format(self.__network_interface.id))
+
+    def reuse_init(self, network_interface_id):
+        '''
+        To reuse an exist network interface than create a new one
+        :params network-intserface_id: id of existing network_interface
+        '''
+        if network_interface_id is None:
+            return False
+        try:
+            self.__network_interface = self.__ec2.NetworkInterface(
+                network_interface_id)
+            if self.is_attached():
+                return False
+            LOG.info(
+                "Existing %s state is %s" %
+                (self.__network_interface.id, self.__network_interface.status))
+            return True
+        except ClientError as err:
+            LOG.error(err)
+            return False
+
+    def is_free(self):
+        self.__network_interface.reload()
+        if self.__network_interface.status == 'in-use':
+            LOG.info("%s network interface is in use.",
+                     self.__network_interface.id)
+            return True
+        return False
+
+    def get_state(self):
+        try:
+            state = 'unknown'
+            self.__network_interface.reload()
+            state = self.__network_interface.status
+            LOG.info("nic state: {}".format(state))
+        except Exception as err:
+            return state
+        return state
+
+    def is_exist(self):
+        """
+        Judge if nic exists
+        :return: True if exists.
+        Example:
+        return True if self._get_status() != -1 else False
+        """
+        try:
+            self.__network_interface.reload()
+            if self.__network_interface.status in "deleting | deleted":
+                LOG.info("nic is deleted %s" % self.id)
+                return False
+            else:
+                LOG.info("nic exists %s" % self.id)
+                return True
+        except Exception as err:
+            LOG.info("nic does not exists %s" % self.id)
+            return False
+
+    def add_tag(self):
+        try:
+            self.__network_interface.reload()
+            self.__network_interface.create_tags(Tags=[
+                {
+                    'Key': 'Name',
+                    'Value': self.tag
+                },
+            ])
+        except Exception as err:
+            LOG.info("Failed to add tag to %s", self.__network_interface.id)
+            LOG.error(err)
+
+    def create(self):
+        '''Create a new network interface
+        '''
+        try:
+            self.__network_interface = self.subnet.create_network_interface(
+                Description=self.tag, Groups=[
+                    self.security_group_ids,
+                ])
+            LOG.info("%s network interface created!" %
+                     self.__network_interface.id)
+            self.add_tag()
+            self.id = self.__network_interface.id
+            return True
+        except Exception as err:
+            LOG.info("Failed to create interface")
+            LOG.error(err)
+            return False
+
+    def delete(self, wait=True):
+        '''
+        Delete network interface
+        :param wait: Wait for interface deleted
+        :return: True|False  and raise Exception if interface delete failed
+        '''
+
+        if self.__network_interface is not None:
+            LOG.info("Delete {}".format(self.id))
+            try:
+                self.__network_interface.delete()
+                return True
+            except Exception as err:
+                LOG.error(err)
+                return False
+        else:
+            LOG.info("No specify network interface delete.")
+
+    def attach_to_instance(self, instance_id, device_index, wait=True):
+        """
+        Attach nic to instance as $device_index
+        :param instance_id: id of instance
+        :param device_index: [0..9]
+        :return: True if success, False as failed
+        """
+        try:
+            LOG.info("Try to attach %s to %s" %
+                     (self.__network_interface.id, instance_id))
+            self.__network_interface.attach(
+                DeviceIndex=device_index,
+                InstanceId=instance_id,
+            )
+
+            if wait:
+                start_time = time.time()
+                while True:
+                    self.__network_interface.reload()
+                    if self.__network_interface.status == 'in-use':
+                        LOG.info('NIC attached!')
+                        return True
+                    else:
+                        end_time = time.time()
+                        if int(end_time) - int(start_time) > 80:
+                            LOG.error(
+                                "Failed to attach to instance after 80s! %s" %
+                                self.__network_interface.status)
+                            return False
+                    time.sleep(10)
+
+            return True
+        except Exception as err:
+            LOG.error("NIC cannot attach to %s error %s" % (instance_id, err))
+            return False
+
+    def detach_from_instance(self, instance_id, wait=True, force=False):
+        """Detach nic from instance as $device_name
+
+        Arguments:
+            instance_id {string} -- instance id
+            device_name {string} -- target device name from instance,
+                                    like 'sdX','xvdx'
+
+        Keyword Arguments:
+            wait {bool} -- wait or not wait detach operation complete
+                           (default: {True})
+            force {bool} -- force or not force detach from instance
+                            (default: {False})
+
+        Returns:
+            bool -- True if success, False as failed
+        """
+        try:
+            LOG.info("Try to detach %s from %s" %
+                     (self.__network_interface.id, instance_id))
+            self.__network_interface.reload()
+            self.__network_interface.detach(Force=force)
+            if wait:
+                start_time = time.time()
+                while True:
+                    self.__network_interface.reload()
+                    if self.__network_interface.status == 'available':
+                        LOG.info('NIC detached!')
+                        return True
+                    else:
+                        end_time = time.time()
+                        if int(end_time) - int(start_time) > 80:
+                            LOG.error(
+                                "Failed to detach from instance after 80s! %s" %
+                                self.__network_interface.status)
+                            return False
+                    time.sleep(10)
+            return True
+        except Exception as err:
+            LOG.error("NIC cannot detach from %s error %s" %
+                      (instance_id, err))
+            return False
