@@ -13,6 +13,7 @@ import difflib
 import time
 import logging
 import argparse
+from tipset.libs import rmt_ssh
 from functools import wraps
 from yaml import load, dump
 try:
@@ -54,8 +55,6 @@ def init_args():
                     help='save result to specific directory', required=False)
     parser.add_argument('--image', dest='image', default=None, action='store',
                     help='specify azure to run azure image check only', required=False)
-    parser.add_argument('--disks', dest='blk_devs', default=None, action='store',
-                    help='free disks for storage test, eg. "/dev/nvme0n1", data on disk has lost risks', required=False)
     parser.add_argument('--platform_profile', dest='platform_profile', default=None, action='store',
                     help='specify platform profile if enable os-tests provison vms self, only supports aws for now', required=False)
     args = parser.parse_args()
@@ -63,12 +62,15 @@ def init_args():
 
 def init_provider(params=None):
     vm = None
+    disk = None
+    nic = None
     if 'aws' in params['Cloud']['provider']:
-        from .resources_aws import EC2VM,EC2Volume
+        from .resources_aws import EC2VM,EC2Volume,EC2NIC
         vm = EC2VM(params)
         vm.create()
         # init disk and do not create disk at very beginning
         disk = EC2Volume(params)
+        nic = EC2NIC(params)
     if 'openstack' in params['Cloud']['provider']:
         from .resources_openstack import OpenstackVM
         vm = OpenstackVM(params)
@@ -81,10 +83,38 @@ def init_provider(params=None):
         if vm.is_stopped():
             vm.start(wait=True)
         disk = None
-    return vm, disk
+    if 'nutanix' in params['Cloud']['provider']:
+        from .resources_nutanix import NutanixVM,NutanixVolume
+        vm = NutanixVM(params)
+        vm.create(wait=True)
+        if vm.is_stopped():
+            vm.start(wait=True)
+        disk = NutanixVolume(params)
+    if 'google' in params['Cloud']['provider']:
+        from .resources_gcp import GCPVM
+        vm = GCPVM(params)
+        vm.create(wait=True)
+        if vm.is_stopped():
+            vm.start(wait=True)
+        disk = None
+    if 'libvirt' in params['Cloud']['provider']:
+        from .resources_libvirt import LibvirtVM
+        vm = LibvirtVM(params)
+        vm.create(wait=True)
+        if vm.is_stopped():
+            vm.start(wait=True)
+        disk = None
+    if 'openshift' in params['Cloud']['provider']:
+        from .resources_openshift import OpenShiftVM
+        vm = OpenShiftVM(params)
+        vm.create(wait=True)
+        if vm.is_stopped():
+            vm.start(wait=True)
+        disk = None
+
+    return vm, disk, nic
 
 def init_ssh(params=None, timeout=600, interval=10, log=None):
-    from tipset.libs import rmt_ssh
     if log is None:
         LOG_FORMAT = '%(levelname)s:%(message)s'
         log = logging.getLogger(__name__)
@@ -120,8 +150,24 @@ def init_connection(test_instance, timeout=600, interval=10):
         test_instance.log.info("connection is not live")
     test_instance.SSH = init_ssh(params=test_instance.params, timeout=timeout, interval=interval, log=test_instance.log)
     if test_instance.SSH is None:
+        if test_instance.vm:
+            try:
+                test_instance.vm.get_console_log()
+            except NotImplementedError:
+                test_instance.log.info("{} not implement this func: get_console_log".format(test_instance.vm.provider))
+
         test_instance.skipTest("Cannot make ssh connection to remote, please check")
 
+def send_ssh_cmd(rmt_node, rmt_user, rmt_password, command):
+    ssh = rmt_ssh.RemoteSSH()
+    ssh.rmt_node = rmt_node
+    ssh.rmt_user = rmt_user
+    ssh.rmt_password = rmt_password
+    ssh.create_connection()
+    status, outputs = ssh.remote_excute(command)
+    ssh.close()
+    return [status,outputs]
+
 def get_cfg(cfg_file = None):
     # Config file
     if not cfg_file:
@@ -173,6 +219,7 @@ def init_case(test_instance):
     logging.basicConfig(level=logging.INFO, format=FORMAT, filename=log_file)
     test_instance.log.info("-"*80)
     test_instance.log.info("Code Repo: {}".format(test_instance.params['code_repo']))
+    test_instance.log.info("Code Version: v{}".format(os_tests.__version__))
     test_instance.log.info("Case ID: {}".format(test_instance.id()))
     test_instance.log.info("Case Doc: {}".format(eval(test_instance.id()).__doc__))
     test_instance.log.info("Case Params:")
@@ -405,10 +452,14 @@ def run_cmd(test_instance,
         except Exception as err:
             test_instance.log.error("Run cmd failed again {}".format(err))
     if status is None and test_instance.vm:
-        test_instance.vm.get_console_log()
+        try:
+            test_instance.vm.get_console_log()
+        except NotImplementedError:
+            test_instance.log.info("{} not implement this func: get_console_log".format(test_instance.vm.provider))
         test_instance.vm.stop()
         test_instance.vm.start()
-    if cursor is not None and cursor in output:
+        test_instance.params['remote_node'] = test_instance.vm.floating_ip
+    if cursor is not None and output is not None and cursor in output:
         output = output[output.index(cursor):]
     if is_log_output:
         test_instance.log.info("CMD ret: {} out:{}".format(status, output))
@@ -432,12 +483,12 @@ def run_cmd(test_instance,
             else:
                 find_list = re.findall('.*{}.*'.format(key_word), output)
             if len(find_list) > 0:
-                test_instance.log.info('expcted "{}" found in "{}"'.format(key_word, ''.join(find_list)))
+                test_instance.log.info('expected "{}" found in "{}"'.format(key_word, ''.join(find_list)))
             else:
                 if output.count('\n') > 5:
-                    test_instance.fail('expcted "{}" not found in output(check debug log as too many lines)'.format(key_word))
+                    test_instance.fail('expected "{}" not found in output(check debug log as too many lines)'.format(key_word))
                 else:
-                    test_instance.fail('expcted "{}" not found in "{}"'.format(key_word,output))
+                    test_instance.fail('expected "{}" not found in "{}"'.format(key_word,output))
     if expect_not_kw is not None:
         for key_word in expect_not_kw.split(','):
             if output.count('\n') > 5:
@@ -445,15 +496,15 @@ def run_cmd(test_instance,
             else:
                 find_list = re.findall('.*{}.*'.format(key_word), output)
             if len(find_list) == 0:
-                test_instance.log.info('Unexpcted "{}" not found in output'.format(key_word))
+                test_instance.log.info('Unexpected "{}" not found in output'.format(key_word))
             else:
                 if output.count('\n') > 5:
-                    test_instance.fail('Unexpcted "{}" found in {}'.format(key_word, ''.join(find_list)))
+                    test_instance.fail('Unexpected "{}" found in {}'.format(key_word, ''.join(find_list)))
                 else:
-                    test_instance.fail('Unexpcted "{}" found in "{}"'.format(key_word,output))
+                    test_instance.fail('Unexpected "{}" found in "{}"'.format(key_word,output))
     if expect_output is not None:
         test_instance.assertEqual(expect_output,
-                         output,
+                         output.rstrip('\n'),
                          msg='exactly expected %s' %
                          (expect_output))
     if cancel_kw is not None:
@@ -531,7 +582,9 @@ def getboottime(test_instance):
     output = run_cmd(test_instance, "systemd-analyze", expect_ret=0)
     boot_time = re.findall("=.*s", output)[0]
     boot_time = boot_time.strip("=\n")
-    boot_time_sec = re.findall('[0-9.]+s', boot_time)[0]
+    boot_time_sec = re.findall('[0-9.]+s', boot_time)
+    # If ends with 'ms', e.g. 1min 76ms, ignore the ms time.
+    boot_time_sec = boot_time_sec[0] if boot_time_sec else '0s'
     boot_time_sec = boot_time_sec.strip('= s')
     if 'min' in boot_time:
         boot_time_min = re.findall('[0-9]+min', boot_time)[0]
@@ -592,7 +645,7 @@ def is_azure(test_instance, action=None):
         azure: return True
         other: return False
     '''
-    output = run_cmd(test_instance, "route -n")
+    output = run_cmd(test_instance, "/usr/sbin/route -n")
     if 'not found' in output:
         return False
     if '168.63.129.16' in output:
@@ -643,6 +696,26 @@ def is_openstack(test_instance, action=None):
         test_instance.log.info("Not an Openstack system.")
     return False
 
+def is_ahv(test_instance, action=None):
+    '''
+    Check whether system is a nutanix ahv system.
+    Arguments:
+        test_instance {Test instance} -- unittest.TestCase instance
+        action {string} -- cancel case if it is not a nutanix ahv system
+    Return:
+        ahv: return True
+        other: return False
+    '''
+    output = run_cmd(test_instance, "sudo cat /sys/devices/virtual/dmi/id/product_*", expect_ret=0)
+    if 'ahv' in output.lower():
+        test_instance.log.info("Nutanix AHV system.")
+        return True
+    else:
+        if action == "cancel":
+            test_instance.skipTest("Cancel it in non Nutanix AHV system.")
+        test_instance.log.info("Not a Nutanix AHV system.")
+    return False
+
 def is_metal(test_instance, action=None):
     '''
     Check whether system is a baremetal system.
@@ -695,7 +768,7 @@ def is_cmd_exist(test_instance, cmd=None, is_install=True, cancel_case=False):
     pkg_find = "sudo yum provides %s" % cmd
     output = run_cmd(test_instance, pkg_find)
     for i in [arch, 'noarch']:
-        pkg_list_tmp = re.findall(".*%s" % i, output)
+        pkg_list_tmp = re.findall(".*\.{}".format(i), output)
         pkg_list = [i for i in pkg_list_tmp if 'Repo' not in i]
         if len(pkg_list) > 0:
             break
@@ -767,7 +840,7 @@ def pkg_install(test_instance, pkg_name=None, pkg_url=None, force=False):
                 cmd = 'sudo yum -y reinstall %s' % pkg_name
             run_cmd(test_instance, cmd, timeout=1200)
 
-        if not is_pkg_installed(test_instance, pkg_name=pkg_name) and pkg_url is not None or force:
+        if not is_pkg_installed(test_instance, pkg_name=pkg_name) and pkg_url is not None and force:
             test_instance.log.info('Install without dependences!')
             cmd = 'sudo rpm -ivh %s --nodeps' % pkg_url
             if force:
@@ -1091,4 +1164,59 @@ def get_value(x={}, key=None, path=None):
                     break
             else:
                 ret = get_value(x=x.get(i), key=key, path=path)
-    return ret
\ No newline at end of file
+    return ret
+
+def get_public_key(client_user=None):
+    """
+    Return a valid string ssh public key for the user executing autoserv or
+    autotest. If there's no DSA or RSA public key, create a RSA keypair with
+    ssh-keygen and return it.
+
+    :param client_user: genenrate the ssh_key for which client
+                        user to login into the server
+    :type client_user: str
+    :returns: a ssh public key
+    :rtype: str
+    """
+
+    if client_user:
+        if os.environ.get('USER') not in ('root', client_user):
+            raise RuntimeError("Can not set ssh-key for OTHER user using"
+                               "non-root account. Permission Denied.")
+        if client_user == 'root':
+            ssh_conf_path = '/root/.ssh'
+        else:
+            ssh_conf_path = '/home/%s/.ssh' % client_user
+    else:
+        ssh_conf_path = os.path.expanduser('~/.ssh')
+        client_user = os.environ.get('USER')
+
+    dsa_public_key_path = os.path.join(ssh_conf_path, 'id_dsa.pub')
+    dsa_private_key_path = os.path.join(ssh_conf_path, 'id_dsa')
+
+    rsa_public_key_path = os.path.join(ssh_conf_path, 'id_rsa.pub')
+    rsa_private_key_path = os.path.join(ssh_conf_path, 'id_rsa')
+
+    has_dsa_keypair = (os.path.isfile(dsa_public_key_path) and
+                       os.path.isfile(dsa_private_key_path))
+    has_rsa_keypair = (os.path.isfile(rsa_public_key_path) and
+                       os.path.isfile(rsa_private_key_path))
+
+    if has_rsa_keypair:
+        logging.info('RSA keypair found, using it')
+        public_key_path = rsa_public_key_path
+
+    elif has_dsa_keypair:
+        logging.info('DSA keypair found, using it')
+        public_key_path = dsa_public_key_path
+
+    else:
+        logging.info('Neither RSA nor DSA keypair found, creating RSA ssh key pair')
+
+        public_key_path = rsa_public_key_path
+
+    public_key = open(public_key_path, 'r')
+    public_key_str = public_key.read()
+    public_key.close()
+    
+    return public_key_str
