@@ -802,23 +802,37 @@ itlb_multihit|sed 's/:/^/' | column -t -s^"
         component:
             sssd
         bugzilla_id:
-            1797973
+            1797973,2027674
         is_customer_case:
             False
         maintainer:
             xiliang@redhat.com
         description:
-            check if system can enable fips successfully
+            check if there is process core dumped during boot.
         key_steps:
             1.enable fips and reboot the system.
             2.dumped core
         expect_result:
             no core dump
         debug_want:
-            redirected output file
+            attach core files under '/var/lib/systemd/coredump/' if report bz
         """
-        # redirect journalctl output to a file as it is not get return
-        # normally in RHEL7
+        cmd = 'sudo ls  /var/lib/systemd/coredump/core*'
+        core_files = utils_lib.run_cmd(self, cmd, msg='check if core file generated')
+        if 'No such file or directory' not in core_files:
+            self.log.info('Please attached core files when repor bugs')
+            for core_file in core_files.split('\n'):
+                core_file = core_file.strip('\n')
+                if not core_file:
+                    continue
+                cmd = 'sudo chmod 766 {}'.format(core_file)
+                utils_lib.run_cmd(self, cmd, expect_ret=0)
+                if self.params['remote_node'] is not None:
+                    self.log.info('retrive {} from remote to {}'.format(core_file, self.log_dir))
+                    self.SSH.get_file(rmt_file=core_file,local_file='{}/debug/{}'.format(self.log_dir,os.path.basename(core_file)))
+                else:
+                    cmd = "cp {} {}/debug/{}".format(core_file, self.log_dir,os.path.basename(core_file) )
+                    utils_lib.run_cmd(self, cmd, msg='save {} to {}'.format(core_file, self.log_dir))
         utils_lib.check_log(self, 'dumped core', skip_words='test_check_journalctl_dumpedcore', rmt_redirect_stdout=True)
 
     def test_check_journalctl_error(self):
@@ -1532,8 +1546,7 @@ in cmdline as bug1859088")
         expected_result:
             No failed service found.
         '''
-        cmd = 'systemctl'
-        utils_lib.run_cmd(self, cmd, expect_ret=0, expect_not_kw='failed')
+        utils_lib.check_log(self, 'failed', log_cmd='systemctl')
 
     def test_check_sysfs_cpu_list(self):
         '''
@@ -1806,9 +1819,13 @@ current_device"
         elif 'VMware' in lscpu_output:
             self.log.info("Found it is a vmware system!")
             self.assertIn('vmware', virt_what_output)
-        elif 'KVM' in lscpu_output and not utils_lib.is_metal(self):
-            self.log.info("Found it is a kvm system!")
-            self.assertIn('kvm', virt_what_output)
+        elif 'KVM' in lscpu_output:
+            if utils_lib.is_ahv(self):
+                self.log.info("Found it is a Nutanix AHV system!")
+                self.assertIn('ahv', virt_what_output)
+            elif not utils_lib.is_metal(self):
+                self.log.info("Found it is a kvm system!")
+                self.assertIn('kvm', virt_what_output)
         elif utils_lib.is_metal(self) and utils_lib.is_aws(self):
             self.log.info("Found it is a aws bare metal system!")
             self.assertEqual('aws', virt_what_output.strip('\n'))
@@ -2020,15 +2037,28 @@ current_device"
         description:
             check if sos report can run without crash
         key_steps:
-            1.sosreport --batch
+            1.sos report --batch
         expect_result:
             no Traceback in result
         debug_want:
             sos_output.txt
         """
+        cmd = 'sudo rm -rf /var/tmp/sos*'
+        utils_lib.run_cmd(self,cmd,msg="clean up old sos report")
         utils_lib.is_cmd_exist(self, cmd='sosreport')
-        cmd = "sosreport --batch"
-        utils_lib.run_cmd(self,cmd,expect_ret=0,msg="test sosreport",timeout=120)
+        cmd = "sudo sosreport --batch"
+        utils_lib.run_cmd(self,cmd,expect_ret=0,msg="test sosreport",timeout=400)
+        cmd = 'sudo ls /var/tmp/sos*.xz'
+        sosfile = utils_lib.run_cmd(self, cmd, expect_ret=0)
+        sosfile = sosfile.strip('\n')
+        cmd = 'sudo chmod 766 {}'.format(sosfile)
+        utils_lib.run_cmd(self, cmd, expect_ret=0)
+        if self.params['remote_node'] is not None:
+            self.log.info('retrive {} from remote to {}'.format(sosfile, self.log_dir))
+            self.SSH.get_file(rmt_file=sosfile,local_file='{}/debug/{}'.format(self.log_dir,os.path.basename(sosfile)))
+        else:
+            cmd = "cp {} {}/debug/{}".format(sosfile, self.log_dir,os.path.basename(sosfile) )
+            utils_lib.run_cmd(self, cmd, msg='save {} to {}'.format(sosfile, self.log_dir))
     def tearDown(self):
         pass
 
