@@ -480,7 +480,7 @@ int main(int argc, char *argv[])
                int x;
                free(&x);
                }
-            3. # gcc -g -o pp test1.c
+            3. # gcc -g -o pp test.c
             4. # ./pp
         expect_result:
             pp crashed and new core file is generated under /var/lib/systemd/coredump
@@ -499,11 +499,9 @@ int main(int argc, char *argv[])
         if 'Red Hat Enterprise Linux' in product_name:
             cmd = 'systemctl show | grep CORE'
             utils_lib.run_cmd(self, cmd, expect_kw='DefaultLimitCORESoft=0,DefaultLimitCORE=infinity', msg='check default limit core setting')
-        utils_lib.run_cmd(self, 'ulimit -c 0', expect_ret=0, msg='test user can change limit core setting')
-        utils_lib.run_cmd(self, 'ulimit -c', expect_kw='0', msg='check result')
-        utils_lib.run_cmd(self, 'ulimit -c unlimited', expect_ret=0, msg='test user can change limit core setting')
-        utils_lib.run_cmd(self, 'ulimit -c', expect_kw='unlimited', msg='check result')
-        utils_lib.run_cmd(self, 'sudo rm -rf /var/lib/systemd/coredump/core*', msg='clean up core files before testing')
+        utils_lib.run_cmd(self, 'ulimit -c 0;ulimit -c', expect_ret=0, expect_kw='0', msg='test user can change limit core setting')
+        utils_lib.run_cmd(self, 'ulimit -c unlimited;ulimit -c', expect_ret=0, expect_kw='unlimited', msg='test user can change limit core setting')
+        utils_lib.run_cmd(self, 'sudo rm -rf /var/lib/systemd/coredump/core.pp*', msg='clean up core files before testing')
         self.cursor = utils_lib.get_cmd_cursor(self, cmd='journalctl -b0', rmt_redirect_stdout=True)
         test_file = '/tmp/test.c'
         utils_lib.is_cmd_exist(self, 'gcc')
@@ -511,8 +509,8 @@ int main(int argc, char *argv[])
         utils_lib.run_cmd(self, cmd, expect_ret=0, msg='generate {}'.format(test_file))
         cmd = "gcc -g -o /tmp/pp {}".format(test_file)
         utils_lib.run_cmd(self, cmd, expect_ret=0)
-        utils_lib.run_cmd(self, '/tmp/pp', msg='run it to trigger core dump')
-        utils_lib.run_cmd(self, 'sudo ls /var/lib/systemd/coredump/core*', expect_ret=0, msg='check core file generated')
+        utils_lib.run_cmd(self, 'ulimit -c unlimited;/tmp/pp', msg='run it to trigger core dump')
+        utils_lib.run_cmd(self, 'sudo ls /var/lib/systemd/coredump/core.pp*', expect_ret=0, msg='check core file generated')
         utils_lib.check_log(self, "error,warn,fail", log_cmd='journalctl -b0', cursor=self.cursor, rmt_redirect_stdout=True)
 
     def test_podman_build_image(self):
@@ -732,12 +730,17 @@ RUN touch /tmp/test.txt
         cmd = "sudo systemctl restart libvirtd"
         utils_lib.run_cmd(self, cmd, cancel_ret='0', msg = "restart libvirtd")
         utils_lib.is_cmd_exist(self, cmd='virsh')
+        self.ssh_timeout = 1200
+        self.log.info('set ssh connection timeout to {}'.format(self.ssh_timeout))
         if utils_lib.is_arch(self, arch='x86_64'):
-            utils_lib.run_cmd(self,
-                   'cat /proc/cmdline',
-                   cancel_kw='intel_iommu=on',
-                   msg='Check boot line')
-
+            boot_param_required = 'intel_iommu=on'
+            out = utils_lib.run_cmd(self, 'cat /proc/cmdline', msg='Check boot line')
+            if boot_param_required not in out:
+                cmd = 'sudo grubby --update-kernel=ALL --args="{}"'.format(boot_param_required)
+                utils_lib.run_cmd(self, cmd, msg="append {} to boot params".format(boot_param_required))
+                utils_lib.run_cmd(self, 'sudo reboot', msg='reboot system under test')
+                time.sleep(10)
+                utils_lib.init_connection(self, timeout=self.ssh_timeout)
         utils_lib.run_cmd(self, 'sudo lspci', msg="get pci list")
         tmp_pci = None
         cmd = "lspci|grep 'Non-Volatile memory'|wc -l"
@@ -856,6 +859,73 @@ if __name__ == "__main__":
         utils_lib.run_cmd(self, 'sudo python3 t.py')
         utils_lib.run_cmd(self, "dmesg", expect_not_kw='Call trace,Call Trace')
 
+
+    def test_wipefs_cdrom(self):
+        """
+        case_tag:
+            util
+        case_name:
+            test_wipefs_return
+        component:
+            util-linux
+        bugzilla_id:
+            2074486
+        is_customer_case:
+            False
+        maintainer:
+            xuazhao@redhat.com
+        description:
+            Check wipefs cannot erase read-only cdrom device
+        key_steps:part
+            1.check if machine have CDROM mounted
+            2.# wipefs -a /dev/sr0
+            3.# echo $?
+            4.#dmesg -T | grep error
+            5.check if there's error and return is 0
+        expect_result:
+            command "wipefs -a /dev/sr0" should return 1
+        debug_want:
+            "strace -o log wipefs -a /dev/srN
+        """
+        cmd = "lsblk | grep sr"
+        all = utils_lib.run_cmd(self,cmd,cancel_kw="sr0",msg="check if machine mounted CDROM").split("\n")
+        for i in all:
+            part = i.split(" ")[0]
+            cmd = "wipefs -a /dev/"+part
+            utils_lib.run_cmd(self,cmd,expect_ret=1,msg="erase signature")
+            cmd = "dmesg -T | grep error"
+            utils_lib.run_cmd(self,cmd,expect_not_kw=part,msg="check if there's error")
+
+    def test_grub2_mkconfig(self):
+        """
+        case_tag:
+            osbuild
+        case_name:
+            test_grub2_mkconfig
+        component:
+            osbuild-composer
+        bugzilla_id:
+            bz2056251
+        is_customer_case:
+            True
+        maintainer:
+            xuazhao@redhat.com
+        description:
+            check if grub2-mkconfig can generate a file with set default="${saved_entry}". Without it, system boots into index 0 kernel only.
+        key_steps:|
+            1.grub2-mkconfig -o /tmp/grub.cfg
+            2.cat /tmp/grub.cfg
+        expect_result:
+            there are {saved_entry} in grub.cfg
+        debug_want:
+            grub.cfg
+        """
+        cmd='rpm -qa "grub2*"'
+        utils_lib.run_cmd(self,cmd,msg="show grub version")
+        cmd="grub2-mkconfig -o /tmp/grub.cfg"
+        utils_lib.run_cmd(self,cmd,expect_kw="done",msg="generate file")
+        cmd='cat /tmp/grub.cfg'
+        utils_lib.run_cmd(self,cmd,msg="check if there is saved_entry",expect_kw="{saved_entry"+"}")
     def test_z_nitro_enclaves(self):
         '''
         case_name:
@@ -980,4 +1050,4 @@ if __name__ == "__main__":
         pass
 
 if __name__ == '__main__':
-    unittest.main()
\ No newline at end of file
+    unittest.main()
